package com.hammingweight.hammockexamples.example1;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;

import javax.microedition.io.HttpConnection;

import jmunit.framework.cldc10.TestCase;

import com.hammingweight.hammock.Hammock;
import com.hammingweight.hammock.mocks.microedition.io.MockHttpConnection;
import com.hammingweight.hammockexamples.example1.GetName;

public class GetNameTest extends TestCase {

	// The Object under Test.
	private GetName getName;

	// The mock object handler.
	private Hammock hammock;

	// The mock object(s).
	private MockHttpConnection mockConn;

	public void setUp() {
		this.hammock = new Hammock();
		this.mockConn = new MockHttpConnection(this.hammock);
		this.getName = new GetName();
	}

	public void testGetNameForId() throws IOException {
		// Expectations.
		this.hammock.setExpectation(
				MockHttpConnection.MTHD_SET_REQUEST_METHOD_$_STRING,
				new Object[] { HttpConnection.POST });
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		DataOutputStream dos = new DataOutputStream(baos);
		this.hammock.setExpectation(
				MockHttpConnection.MTHD_OPEN_DATA_OUTPUT_STREAM)
				.setReturnValue(dos);
		ByteArrayOutputStream baos2 = new ByteArrayOutputStream();
		new DataOutputStream(baos2).writeUTF("Carl Meijer");
		this.hammock.setExpectation(
				MockHttpConnection.MTHD_OPEN_DATA_INPUT_STREAM).setReturnValue(
				new DataInputStream(new ByteArrayInputStream(baos2
						.toByteArray())));
		this.hammock.setExpectation(MockHttpConnection.MTHD_CLOSE);

		// Exercise.
		assertEquals("Carl Meijer", this.getName.getNameForId(this.mockConn,
				1234));
		DataInputStream dis = new DataInputStream(new ByteArrayInputStream(baos
				.toByteArray()));
		assertEquals(1234, dis.readInt());

		// Verify.
		this.hammock.verify();
	}

	public void testCloseAfterIoException() {
		// Expectations.
		this.hammock.setExpectation(
				MockHttpConnection.MTHD_SET_REQUEST_METHOD_$_STRING)
				.setThrowable(new IOException());
		this.hammock.setExpectation(MockHttpConnection.MTHD_CLOSE);
		this.hammock.setStrictOrdering();
		
		// Exercise.
		try {
			this.getName.getNameForId(this.mockConn, 1234);
			fail("IOException showld have been thrown");
		} catch (IOException ioe) {
			// Expected.
		}

		// Verify.
		this.hammock.verify();
	}

	public GetNameTest() {
		super(2, "GetNameTest");
	}

	public void test(int testNum) throws Throwable {
		switch (testNum) {
		case 0:
			testGetNameForId();
			break;

		case 1:
			testCloseAfterIoException();
			break;

		default:
			fail("No such test.");
		}
	}
}
