package com.hammingweight.hammockexamples.example4;

import java.io.IOException;

import jmunit.framework.cldc10.TestCase;

import com.hammingweight.hammock.Hammock;
import com.hammingweight.hammock.mocks.io.MockDataInput;
import com.hammingweight.hammock.mocks.io.MockDataOutput;
import com.hammingweight.hammockexamples.example4.GetName2;
import com.hammingweight.hammockexamples.mocks.MockConnectionFactory;

public class GetName2Test extends TestCase {

	// The OUT.
	private GetName2 getName;

	// The handler and mock(s)
	private Hammock hammock;
	private MockConnectionFactory mockConnFactory;
	private MockDataOutput mockDataOutput;
	private MockDataInput mockDataInput;

	public void setUp() {
		this.hammock = new Hammock();
		this.mockConnFactory = new MockConnectionFactory(this.hammock);
		this.mockDataOutput = new MockDataOutput(this.hammock);
		this.mockDataInput = new MockDataInput(this.hammock);
		this.getName = new GetName2(this.mockConnFactory);
	}

	public void testGetNameForId() throws IOException {
		// Expectations.
		this.hammock.setExpectation(
				MockConnectionFactory.MTHD_OPEN_CONNECTIONS_$_STRING,
				new Object[] { "getname.inm" });
		this.hammock.setExpectation(
				MockConnectionFactory.MTHD_GET_OUTPUT_CONNECTION)
				.setReturnValue(this.mockDataOutput);
		this.hammock.setExpectation(MockDataOutput.MTHD_WRITE_INT_$_INT,
				new Object[] { new Integer(1234) });
		this.hammock.setExpectation(
				MockConnectionFactory.MTHD_GET_INPUT_CONNECTION)
				.setReturnValue(this.mockDataInput);
		this.hammock.setExpectation(MockDataInput.MTHD_READ_UTF)
				.setReturnValue("Carl Meijer");
		this.hammock.setExpectation(MockConnectionFactory.MTHD_CLOSE_CONNECTIONS);

		// Exercise.
		assertEquals("Carl Meijer", this.getName.getNameForId(1234));

		// Verify.
		this.hammock.verify();
	}

	public void testClose() {
		// Expectations.
		this.hammock.setExpectation(
				MockConnectionFactory.MTHD_OPEN_CONNECTIONS_$_STRING)
				.setThrowable(new IOException());
		this.hammock.setExpectation(MockConnectionFactory.MTHD_CLOSE_CONNECTIONS);

		// Exercise.
		try {
			this.getName.getNameForId(1234);
			fail("AN IOException should have been thrown.");
		} catch (IOException e) {
			// Expected.
		}

		// Verify.
		this.hammock.verify();
	}

	public GetName2Test() {
		super(2, "GetName2Test");
	}

	public void test(int testNum) throws Throwable {
		switch (testNum) {
		case 0:
			testGetNameForId();
			break;

		case 1:
			testClose();
			break;

		default:
			fail("No such test.");
		}
	}

}
