/*
 * Methods.java
 *  
 * Copyright 2007 C.A. Meijer
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.hammingweight.hammockmaker;

import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.Collection;
import java.util.Iterator;
import java.util.Map;
import java.util.TreeMap;
import java.util.Vector;

/**
 *
 * @author Carl
 */
public class Methods implements Iterable<MethodWrapper> {

   private Map<String, MethodWrapper> methodWrappers;
   
   private static boolean shouldIgnoreMethod(Method method) throws Exception {
      if (method.getName().equals("equals")) {
         if ((method.getParameterTypes().length == 1) && 
               (!method.getParameterTypes()[0].isPrimitive())) {
            return true;
         }
      }
      
      if (method.getName().equals("hashCode")) {
         if (method.getParameterTypes().length == 0) {
            return true;
         }
      }
      
      if (method.getName().equals("toString")) {
         if (method.getParameterTypes().length == 0) {
            return true;
         }
      }
      
      int modifiers = method.getModifiers();
      if (Modifier.isStatic(modifiers)) {
         return true;
      }
      
      if (Modifier.isVolatile(modifiers)) {
         return true;
      }
      
      if (Modifier.isPublic(modifiers) || Modifier.isProtected(modifiers)) {
         return false;
      }
      
      return true;
   }
   
   private static void addMethods(Collection<Method> methodSet, Method[] methods) {
      for (Method method : methods) {
         methodSet.add(method);
      }
   }
   
   /** Creates a new instance of Methods */
   public Methods(Class<?> clazz) throws Exception {
      methodWrappers = new TreeMap<String, MethodWrapper>();
      
      Vector<Method> methodSet = new Vector<Method>();
      while ((clazz != Object.class) && (clazz != null)) {
         addMethods(methodSet, clazz.getMethods());
         addMethods(methodSet, clazz.getDeclaredMethods());
         clazz = clazz.getSuperclass();
      }
      
      for (Method method : methodSet) {
         String methodId = MethodWrapper.getMethodIdentifier(method);
         MethodWrapper methodWrapper = new MethodWrapper(methodId, method);
         if (methodWrappers.containsValue(methodWrapper)) {
           continue;
         }

         if (shouldIgnoreMethod(method)) {
            continue;
         }

         if (methodWrappers.containsKey(methodId)) {
            int i = 2;
            String methodIdCandidate;
            do {
               methodIdCandidate = methodId + Integer.toString(i);
            } while (methodWrappers.containsKey(methodIdCandidate));
            methodId = methodIdCandidate;
         }
         methodWrappers.put(methodId, methodWrapper);
      }
      
      for (String methodId : methodWrappers.keySet()) {
    	  MethodWrapper mw = this.methodWrappers.get(methodId);
    	  mw.setMethodId(methodId);
      }
   }

   public Iterator<MethodWrapper> iterator() {
      return methodWrappers.values().iterator();
   }
}
